pub struct SpriteOps;Expand description
Zero-sized type hosting Sprite factory operations.
Implementations
Sourceimpl SpriteOps
impl SpriteOps
Sourcepub fn new_sprite(texture_handle: u64) -> FfiSprite
pub fn new_sprite(texture_handle: u64) -> FfiSprite
Creates a new sprite with a texture handle.
Sourcepub fn new_default() -> FfiSprite
pub fn new_default() -> FfiSprite
Creates a default sprite with an invalid texture handle.
Sourcepub fn with_color(
sprite: FfiSprite,
r: f32,
g: f32,
b: f32,
a: f32,
) -> FfiSprite
pub fn with_color( sprite: FfiSprite, r: f32, g: f32, b: f32, a: f32, ) -> FfiSprite
Creates a new sprite with the specified color tint.
Sourcepub fn with_source_rect(
sprite: FfiSprite,
x: f32,
y: f32,
width: f32,
height: f32,
) -> FfiSprite
pub fn with_source_rect( sprite: FfiSprite, x: f32, y: f32, width: f32, height: f32, ) -> FfiSprite
Creates a new sprite with the specified source rectangle.
Sourcepub fn with_flip_x(sprite: FfiSprite, flip: bool) -> FfiSprite
pub fn with_flip_x(sprite: FfiSprite, flip: bool) -> FfiSprite
Creates a new sprite with the specified horizontal flip.
Sourcepub fn with_flip_y(sprite: FfiSprite, flip: bool) -> FfiSprite
pub fn with_flip_y(sprite: FfiSprite, flip: bool) -> FfiSprite
Creates a new sprite with the specified vertical flip.
Sourcepub fn with_flip(sprite: FfiSprite, flip_x: bool, flip_y: bool) -> FfiSprite
pub fn with_flip(sprite: FfiSprite, flip_x: bool, flip_y: bool) -> FfiSprite
Creates a new sprite with both flip flags set.