Transform2DPtrOps

Struct Transform2DPtrOps 

pub struct Transform2DPtrOps;
Zero-sized type for Transform2D pointer operations.

Implementations

Source

impl Transform2DPtrOps

pub fn translate(transform: *mut FfiTransform2D, dx: f32, dy: f32)

Translates the transform by the given offset.

pub fn translate_local(transform: *mut FfiTransform2D, dx: f32, dy: f32)

Translates the transform in local space.

pub fn set_position(transform: *mut FfiTransform2D, x: f32, y: f32)

Sets the position of the transform.

pub fn get_position(transform: *const FfiTransform2D) -> FfiVec2

Gets the position of the transform.

pub fn rotate(transform: *mut FfiTransform2D, angle: f32)

Rotates the transform by the given angle in radians.

pub fn rotate_degrees(transform: *mut FfiTransform2D, degrees: f32)

Rotates the transform by the given angle in degrees.

pub fn set_rotation(transform: *mut FfiTransform2D, rotation: f32)

Sets the rotation angle in radians.

pub fn set_rotation_degrees(transform: *mut FfiTransform2D, degrees: f32)

Sets the rotation angle in degrees.

pub fn get_rotation(transform: *const FfiTransform2D) -> f32

Gets the rotation angle in radians.

pub fn get_rotation_degrees(transform: *const FfiTransform2D) -> f32

Gets the rotation angle in degrees.

pub fn look_at_target( transform: *mut FfiTransform2D, target_x: f32, target_y: f32, )

Makes the transform look at a target position.

pub fn set_scale(transform: *mut FfiTransform2D, scale_x: f32, scale_y: f32)

Sets the scale of the transform.

pub fn set_scale_uniform(transform: *mut FfiTransform2D, scale: f32)

Sets uniform scale on both axes.

pub fn get_scale(transform: *const FfiTransform2D) -> FfiVec2

Gets the scale of the transform.

pub fn scale_by(transform: *mut FfiTransform2D, factor_x: f32, factor_y: f32)

Multiplies the current scale by the given factors.

pub fn forward(transform: *const FfiTransform2D) -> FfiVec2

Returns the forward direction vector.

pub fn right(transform: *const FfiTransform2D) -> FfiVec2

Returns the right direction vector.

pub fn backward(transform: *const FfiTransform2D) -> FfiVec2

Returns the backward direction vector.

pub fn left(transform: *const FfiTransform2D) -> FfiVec2

Returns the left direction vector.

pub fn matrix(transform: *const FfiTransform2D) -> FfiMat3x3

Computes the 3x3 transformation matrix.

pub fn matrix_inverse(transform: *const FfiTransform2D) -> FfiMat3x3

Computes the inverse transformation matrix.

pub fn transform_point( transform: *const FfiTransform2D, point_x: f32, point_y: f32, ) -> FfiVec2

Transforms a point from local space to world space.

pub fn transform_direction( transform: *const FfiTransform2D, dir_x: f32, dir_y: f32, ) -> FfiVec2

Transforms a direction from local space to world space.

pub fn inverse_transform_point( transform: *const FfiTransform2D, point_x: f32, point_y: f32, ) -> FfiVec2

Transforms a point from world space to local space.

pub fn inverse_transform_direction( transform: *const FfiTransform2D, dir_x: f32, dir_y: f32, ) -> FfiVec2

Transforms a direction from world space to local space.