Parent

Struct Parent 

pub struct Parent { /* private fields */ }
Component indicating the parent entity of this entity.

When an entity has a Parent component, its transform (if any) is considered to be in the parent’s local coordinate space. The hierarchy propagation system will compute the global transform by combining the parent’s global transform with this entity’s local transform.

Memory Layout

Parent (8 bytes total):
┌────────────────┬────────────────┐
│  index (u32)   │ generation(u32)│  <- Entity
└────────────────┴────────────────┘

Example

use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;

let parent_entity = Entity::new(0, 1);
let parent_component = Parent::new(parent_entity);

assert_eq!(parent_component.get(), parent_entity);

Implementations

Source

impl Parent

pub const fn new(parent: Entity) -> Parent

Creates a new Parent component pointing to the given entity.

Arguments
  • parent - The entity that should be this entity’s parent
Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;

let parent_entity = Entity::new(42, 1);
let parent = Parent::new(parent_entity);
pub const fn get(&self) -> Entity

Returns the parent entity.

Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;

let parent_entity = Entity::new(10, 2);
let parent = Parent::new(parent_entity);

assert_eq!(parent.get(), parent_entity);
pub fn set(&mut self, parent: Entity)

Sets the parent entity.

This allows changing the parent without removing and re-adding the component.

Arguments
  • parent - The new parent entity
Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;

let mut parent = Parent::new(Entity::new(0, 1));
parent.set(Entity::new(5, 1));

assert_eq!(parent.get(), Entity::new(5, 1));