#[repr(C)]pub struct Vec4 {
pub x: f32,
pub y: f32,
pub z: f32,
pub w: f32,
}Expand description
A 4D vector with FFI-safe memory layout.
This type is commonly used for homogeneous coordinates in graphics or RGBA colors. It is guaranteed to have the same memory layout as a C struct with four consecutive f32 fields.
Fields
x: f32The x-component of the vector.
y: f32The y-component of the vector.
z: f32The z-component of the vector.
w: f32The w-component of the vector.
Implementations