pub enum InputBinding {
Key(Key),
MouseButton(MouseButton),
GamepadButton {
gamepad_id: usize,
button: u32,
},
}Expand description
Represents a single input binding that can be mapped to an action.
An action can have multiple bindings, allowing for keyboard, mouse, and gamepad inputs to all trigger the same action.
Variants
Key(Key)
A keyboard key.
MouseButton(MouseButton)
A mouse button.
GamepadButton
A gamepad button for a specific gamepad.
Implementations
Sourceimpl InputBinding
impl InputBinding
Sourcepub fn is_pressed(&self, input: &InputManager) -> bool
pub fn is_pressed(&self, input: &InputManager) -> bool
Returns true if this binding is currently pressed.
Sourcepub fn is_just_pressed(&self, input: &InputManager) -> bool
pub fn is_just_pressed(&self, input: &InputManager) -> bool
Returns true if this binding was just pressed this frame.
Sourcepub fn is_just_released(&self, input: &InputManager) -> bool
pub fn is_just_released(&self, input: &InputManager) -> bool
Returns true if this binding was just released this frame.