pub struct Parent { /* private fields */ }Expand description
Component indicating the parent entity of this entity.
When an entity has a Parent component, its transform (if any) is
considered to be in the parent’s local coordinate space. The hierarchy
propagation system will compute the global transform by combining
the parent’s global transform with this entity’s local transform.
Memory Layout
Parent (8 bytes total):
┌────────────────┬────────────────┐
│ index (u32) │ generation(u32)│ <- Entity
└────────────────┴────────────────┘Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;
let parent_entity = Entity::new(0, 1);
let parent_component = Parent::new(parent_entity);
assert_eq!(parent_component.get(), parent_entity);Implementations
Sourceimpl Parent
impl Parent
Sourcepub const fn get(&self) -> Entity
pub const fn get(&self) -> Entity
Returns the parent entity.
Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;
let parent_entity = Entity::new(10, 2);
let parent = Parent::new(parent_entity);
assert_eq!(parent.get(), parent_entity);Sourcepub fn set(&mut self, parent: Entity)
pub fn set(&mut self, parent: Entity)
Sets the parent entity.
This allows changing the parent without removing and re-adding the component.
Arguments
parent- The new parent entity
Example
use goud_engine::ecs::Entity;
use goud_engine::ecs::components::Parent;
let mut parent = Parent::new(Entity::new(0, 1));
parent.set(Entity::new(5, 1));
assert_eq!(parent.get(), Entity::new(5, 1));Trait Implementations
Sourceimpl<'de> Deserialize<'de> for Parent
impl<'de> Deserialize<'de> for Parent
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<Parent, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<Parent, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl Serialize for Parent
impl Serialize for Parent
Sourcefn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
fn serialize<__S>(
&self,
__serializer: __S,
) -> Result<<__S as Serializer>::Ok, <__S as Serializer>::Error>where
__S: Serializer,
Serialize this value into the given Serde serializer. Read more
impl Component for Parent
impl Copy for Parent
impl Eq for Parent
impl StructuralPartialEq for Parent